@@ -14,28 +14,23 @@ RUN_TESTS=${RUN_TESTS:-1}
 RUN_MEMLEAK_TEST=${RUN_MEMLEAK_TEST:-0}
 VENDOR_GTEST=${VENDOR_GTEST:-OFF}
 CI_TIMEOUT=${CI_TIMEOUT:-0}
-BUILD_LIBBPF=${BUILD_LIBBPF:-OFF}
 CC=${CC:cc}
 CXX=${CXX:c++}
+ENABLE_SKB_OUTPUT=${ENABLE_SKB_OUTPUT:-ON}
+USE_SYSTEM_BPF_BCC=${USE_SYSTEM_BPF_BCC:-OFF}
 
 if [[ $LLVM_VERSION -eq 13 ]]; then 
   touch /usr/lib/llvm-13/bin/llvm-omp-device-info
 fi
 
-if [[ $BUILD_LIBBPF = ON ]]; then
-  mkdir -p /src
-  git clone https://github.com/libbpf/libbpf.git /src/libbpf
-  cd /src/libbpf/src
-  git checkout v0.5.0
-  CC=gcc make -j$(nproc)
-  # libbpf defaults to /usr/lib64 which doesn't work on debian like systems
-  # this should work on both
-  PREFIX=/usr/local/ LIBDIR=/usr/local/lib make install install_uapi_headers
-fi
 
-# Build bpftrace
 mkdir -p "$1"
 cd "$1"
+
+# Build vendored libraries first
+../build-libs.sh
+
+# Build bpftrace
 cmake -DCMAKE_BUILD_TYPE="$2" \
       -DWARNINGS_AS_ERRORS:BOOL=$WARNINGS_AS_ERRORS \
       -DSTATIC_LINKING:BOOL=$STATIC_LINKING \
@@ -47,6 +42,8 @@ cmake -DCMAKE_BUILD_TYPE="$2" \
       -DVENDOR_GTEST=$VENDOR_GTEST \
       -DBUILD_ASAN:BOOL=$RUN_MEMLEAK_TEST \
       -DCMAKE_VERBOSE_MAKEFILE:BOOL=ON \
+      -DENABLE_SKB_OUTPUT:BOOL=$ENABLE_SKB_OUTPUT \
+      -DUSE_SYSTEM_BPF_BCC:BOOL=$USE_SYSTEM_BPF_BCC \
       "${CMAKE_EXTRA_FLAGS}" \
       ../
 shift 2
