@@ -1,8 +1,10 @@
-#include "ast.h"
-#include "log.h"
-#include "visitors.h"
+#include "ast/ast.h"
+
 #include <iostream>
 
+#include "ast/visitors.h"
+#include "log.h"
+
 namespace bpftrace {
 namespace ast {
 
@@ -624,6 +626,18 @@ Probe::Probe(const Probe &other) : Node(other)
   index_ = other.index_;
 }
 
+bool Probe::has_ap_of_probetype(ProbeType probe_type)
+{
+  if (!attach_points)
+    return false;
+  for (auto ap : *attach_points)
+  {
+    if (probetype(ap->provider) == probe_type)
+      return true;
+  }
+  return false;
+}
+
 While::While(const While &other) : Statement(other)
 {
 }
