@@ -1,4 +1,4 @@
-#include "attachpoint_parser.h"
+#include "ast/attachpoint_parser.h"
 
 #include <algorithm>
 #include <exception>
@@ -6,15 +6,11 @@
 #include <string>
 #include <unordered_map>
 #include <vector>
-
-#include "int_parser.h"
+#include <bcc/bcc_proc.h>
+#include "ast/int_parser.h"
 #include "log.h"
 #include "types.h"
 
-#ifdef HAVE_BCC_WHICH_SO
-#include <bcc/bcc_proc.h>
-#endif
-
 namespace bpftrace {
 namespace ast {
 
@@ -86,6 +82,19 @@ int AttachPointParser::parse()
         }
       }
     }
+
+    auto new_end = std::remove_if(probe->attach_points->begin(),
+                                  probe->attach_points->end(),
+                                  [](const AttachPoint *ap) {
+                                    return ap->provider.empty();
+                                  });
+    probe->attach_points->erase(new_end, probe->attach_points->end());
+
+    if (probe->attach_points->empty())
+    {
+      LOG(ERROR, probe->loc, sink_) << "No attach points for probe";
+      failed++;
+    }
   }
 
   return failed;
@@ -105,6 +114,13 @@ AttachPointParser::State AttachPointParser::parse_attachpoint(AttachPoint &ap)
     return INVALID;
   }
 
+  if (parts_.front().empty())
+  {
+    // Do not fail on empty attach point, could be just a trailing comma
+    ap_->provider = "";
+    return OK;
+  }
+
   std::set<std::string> probe_types;
   if (has_wildcard(parts_.front()))
   {
@@ -160,6 +176,11 @@ AttachPointParser::State AttachPointParser::parse_attachpoint(AttachPoint &ap)
 
   switch (probetype(ap.provider))
   {
+    case ProbeType::invalid:
+      LOG(FATAL) << "Invalid probe type made it to attachpoint parser";
+      return INVALID;
+    case ProbeType::special:
+      return special_parser();
     case ProbeType::kprobe:
       return kprobe_parser();
     case ProbeType::kretprobe:
@@ -189,9 +210,6 @@ AttachPointParser::State AttachPointParser::parse_attachpoint(AttachPoint &ap)
       return kfunc_parser();
     case ProbeType::iter:
       return iter_parser();
-    default:
-      errs_ << "Unrecognized probe type: " << ap_->provider << std::endl;
-      return INVALID;
   }
 
   return OK;
@@ -224,7 +242,7 @@ AttachPointParser::State AttachPointParser::lex_attachpoint(
     }
     else if (!in_quotes && raw[idx] == '$')
     {
-      // There's an assumption that the positional paramter is well
+      // There's an assumption that the positional parameter is well
       // formed. ie we are not expecting a bare `$` or `$nonint`. The
       // bison parser should have guaranteed this.
       size_t i = idx + 1;
@@ -266,6 +284,22 @@ AttachPointParser::State AttachPointParser::lex_attachpoint(
   return State::OK;
 }
 
+AttachPointParser::State AttachPointParser::special_parser()
+{
+  // Can only have reached here if provider is `BEGIN` or `END`
+  assert(ap_->provider == "BEGIN" || ap_->provider == "END");
+
+  if (parts_.size() == 2 && parts_[1] == "*")
+    parts_.pop_back();
+  if (parts_.size() != 1)
+  {
+    errs_ << ap_->provider << " probe type requires 0 arguments" << std::endl;
+    return INVALID;
+  }
+
+  return OK;
+}
+
 AttachPointParser::State AttachPointParser::kprobe_parser(bool allow_offset)
 {
   if (parts_.size() != 2)
@@ -332,26 +366,12 @@ AttachPointParser::State AttachPointParser::kretprobe_parser()
 AttachPointParser::State AttachPointParser::uprobe_parser(bool allow_offset,
                                                           bool allow_abs_addr)
 {
-  // Handle special probes implemented as uprobes
-  if (ap_->provider == "BEGIN" || ap_->provider == "END")
-  {
-    if (parts_.size() == 2 && parts_[1] == "*")
-      parts_.pop_back();
-    if (parts_.size() != 1)
-    {
-      errs_ << ap_->provider << " probe type requires 0 arguments" << std::endl;
-      return INVALID;
-    }
-
-    return OK;
-  }
-
-  // Now handle regular uprobes
   if (parts_.size() == 2 && bpftrace_.pid() > 0)
   {
     // For PID, the target may be skipped
     parts_.push_back(parts_[1]);
-    parts_[1] = "";
+    auto target = get_pid_exe(bpftrace_.pid());
+    parts_[1] = path_for_pid_mountns(bpftrace_.pid(), target);
   }
   if (parts_.size() != 3)
   {
@@ -364,7 +384,6 @@ AttachPointParser::State AttachPointParser::uprobe_parser(bool allow_offset,
 
   ap_->target = "";
 
-#ifdef HAVE_BCC_WHICH_SO
   if (!has_wildcard(parts_[1]) && parts_[1].find("lib") == 0)
   {
     // Automatic resolution of shared library paths.
@@ -376,17 +395,10 @@ AttachPointParser::State AttachPointParser::uprobe_parser(bool allow_offset,
     if (lib_path)
       ap_->target = lib_path;
   }
-#endif
 
   if (ap_->target.empty())
   {
-    if (bpftrace_.pid() > 0)
-    {
-      ap_->target = get_pid_exe(bpftrace_.pid());
-      ap_->target = path_for_pid_mountns(bpftrace_.pid(), ap_->target);
-    }
-    else
-      ap_->target = parts_[1];
+    ap_->target = parts_[1];
   }
 
   // Handle uprobe:/lib/asdf:func+0x100 case
@@ -432,6 +444,11 @@ AttachPointParser::State AttachPointParser::uprobe_parser(bool allow_offset,
       auto res = stoll(parts_[2]);
       if (res)
       {
+        if (has_wildcard(ap_->target))
+        {
+          errs_ << "Cannot use wildcards with absolute address" << std::endl;
+          return INVALID;
+        }
         ap_->address = *res;
       }
       else
@@ -674,19 +691,53 @@ AttachPointParser::State AttachPointParser::watchpoint_parser(bool async)
 
 AttachPointParser::State AttachPointParser::kfunc_parser()
 {
-  if (parts_.size() != 2)
+  // kfunc[:module]:function
+  if (parts_.size() != 2 && parts_.size() != 3)
   {
     if (ap_->ignore_invalid)
       return SKIP;
 
-    errs_ << ap_->provider << " probe type requires 1 argument" << std::endl;
+    errs_ << ap_->provider << " probe type requires 1 or 2 arguments"
+          << std::endl;
     return INVALID;
   }
 
-  if (parts_[1].find('*') != std::string::npos)
+  if (parts_.size() == 3)
+  {
+    ap_->target = parts_[1];
+    ap_->func = parts_[2];
+  }
+  else
+  {
+    ap_->func = parts_[1];
+    if (ap_->func.find('*') == std::string::npos)
+    {
+      auto func_modules = bpftrace_.get_func_modules(ap_->func);
+      if (func_modules.size() == 1)
+        ap_->target = *func_modules.begin();
+      else if (func_modules.size() > 1)
+      {
+        if (listing_)
+          ap_->target = "*";
+        else
+        {
+          // Attaching to multiple functions of the same name is currently
+          // broken, ask the user to specify a module explicitly.
+          errs_ << "ambiguous attach point, please specify module containing "
+                   "the function \'"
+                << ap_->func << "\'";
+          return INVALID;
+        }
+      }
+    }
+    else // leave the module empty for now
+      ap_->target = "*";
+  }
+
+  if (ap_->func.find('*') != std::string::npos ||
+      ap_->target.find('*') != std::string::npos)
     ap_->need_expansion = true;
 
-  ap_->func = parts_[1];
   return OK;
 }
 
