@@ -1,9 +1,11 @@
 #include "field_analyser.h"
+
+#include <cassert>
+#include <iostream>
+
 #include "dwarf_parser.h"
 #include "log.h"
 #include "probe_matcher.h"
-#include <cassert>
-#include <iostream>
 
 namespace bpftrace {
 namespace ast {
@@ -15,15 +17,17 @@ void FieldAnalyser::visit(Identifier &identifier)
 
 void FieldAnalyser::visit(Builtin &builtin)
 {
+  std::string builtin_type;
+  sized_type_ = CreateNone();
   if (builtin.ident == "ctx")
   {
     switch (prog_type_)
     {
-      case BPF_PROG_TYPE_KPROBE:
-        bpftrace_.btf_set_.insert("struct pt_regs");
+      case libbpf::BPF_PROG_TYPE_KPROBE:
+        builtin_type = "struct pt_regs";
         break;
-      case BPF_PROG_TYPE_PERF_EVENT:
-        bpftrace_.btf_set_.insert("struct bpf_perf_event_data");
+      case libbpf::BPF_PROG_TYPE_PERF_EVENT:
+        builtin_type = "struct bpf_perf_event_data";
         break;
       default:
         break;
@@ -32,52 +36,34 @@ void FieldAnalyser::visit(Builtin &builtin)
     // make them resolved and available
     if (probe_type_ == ProbeType::iter)
     {
-      std::string it_struct;
-
       if (attach_func_ == "task")
-      {
-        it_struct = "struct bpf_iter__task";
-      }
+        builtin_type = "struct bpf_iter__task";
       else if (attach_func_ == "task_file")
-      {
-        it_struct = "struct bpf_iter__task_file";
-      }
-
-      if (!it_struct.empty())
-      {
-        bpftrace_.btf_set_.insert(it_struct);
-        type_ = it_struct;
-      }
+        builtin_type = "struct bpf_iter__task_file";
     }
   }
   else if (builtin.ident == "curtask")
   {
-    type_ = "struct task_struct";
-    bpftrace_.btf_set_.insert(type_);
+    builtin_type = "struct task_struct";
   }
   else if (builtin.ident == "args")
   {
     resolve_args(*probe_);
     has_builtin_args_ = true;
+    return;
   }
   else if (builtin.ident == "retval")
   {
     resolve_args(*probe_);
 
     auto it = ap_args_.find("$retval");
-
     if (it != ap_args_.end())
-    {
-      if (it->second.IsRecordTy())
-        type_ = it->second.GetName();
-      else if (it->second.IsPtrTy() && it->second.GetPointeeTy()->IsRecordTy())
-        type_ = it->second.GetPointeeTy()->GetName();
-      else
-        type_ = "";
-    }
-
-    bpftrace_.btf_set_.insert(type_);
+      sized_type_ = it->second;
+    return;
   }
+
+  if (bpftrace_.has_btf_data())
+    sized_type_ = bpftrace_.btf_->get_stype(builtin_type);
 }
 
 void FieldAnalyser::visit(Map &map)
@@ -91,14 +77,14 @@ void FieldAnalyser::visit(Map &map)
 
   auto it = var_types_.find(map.ident);
   if (it != var_types_.end())
-    type_ = it->second;
+    sized_type_ = it->second;
 }
 
 void FieldAnalyser::visit(Variable &var __attribute__((unused)))
 {
   auto it = var_types_.find(var.ident);
   if (it != var_types_.end())
-    type_ = it->second;
+    sized_type_ = it->second;
 }
 
 void FieldAnalyser::visit(FieldAccess &acc)
@@ -110,51 +96,76 @@ void FieldAnalyser::visit(FieldAccess &acc)
   if (has_builtin_args_)
   {
     auto it = ap_args_.find(acc.field);
-
     if (it != ap_args_.end())
-    {
-      if (it->second.IsRecordTy())
-        type_ = it->second.GetName();
-      else if (it->second.IsPtrTy() && it->second.GetPointeeTy()->IsRecordTy())
-        type_ = it->second.GetPointeeTy()->GetName();
-      else
-        type_ = "";
-    }
+      sized_type_ = it->second;
 
-    bpftrace_.btf_set_.insert(type_);
     has_builtin_args_ = false;
   }
-  else if (!type_.empty())
+  else if (!sized_type_.IsNoneTy())
   {
-    type_ = bpftrace_.btf_.type_of(type_, acc.field);
-    bpftrace_.btf_set_.insert(type_);
+    if (sized_type_.IsPtrTy())
+    {
+      sized_type_ = *sized_type_.GetPointeeTy();
+      resolve_fields(sized_type_);
+    }
+
+    // If the struct type or the field type has not been resolved, add the type
+    // to the BTF set to let ClangParser resolve it
+    if (bpftrace_.has_btf_data() && sized_type_.IsRecordTy())
+    {
+      SizedType field_type = CreateNone();
+      if (sized_type_.HasField(acc.field))
+        field_type = sized_type_.GetField(acc.field).type;
+
+      if (!field_type.IsNoneTy())
+        sized_type_ = field_type;
+      else
+      {
+        bpftrace_.btf_set_.insert(sized_type_.GetName());
+        auto field_type_name = bpftrace_.btf_->type_of(sized_type_.GetName(),
+                                                       acc.field);
+        bpftrace_.btf_set_.insert(field_type_name);
+      }
+    }
   }
 }
 
 void FieldAnalyser::visit(Cast &cast)
 {
   Visit(*cast.expr);
-  type_ = cast.cast_type;
-  assert(!type_.empty());
-  bpftrace_.btf_set_.insert(type_);
+  sized_type_ = CreateNone();
+
+  for (auto &ap : *probe_->attach_points)
+    if (Dwarf *dwarf = bpftrace_.get_dwarf(*ap))
+      sized_type_ = dwarf->get_stype(cast.cast_type);
+
+  if (sized_type_.IsNoneTy() && bpftrace_.has_btf_data())
+    sized_type_ = bpftrace_.btf_->get_stype(cast.cast_type);
+
+  // Could not resolve destination type - let ClangParser do it
+  if (sized_type_.IsNoneTy())
+    bpftrace_.btf_set_.insert(cast.cast_type);
 }
 
 void FieldAnalyser::visit(AssignMapStatement &assignment)
 {
   Visit(*assignment.map);
   Visit(*assignment.expr);
-  var_types_.emplace(assignment.map->ident, type_);
+  var_types_.emplace(assignment.map->ident, sized_type_);
 }
 
 void FieldAnalyser::visit(AssignVarStatement &assignment)
 {
   Visit(*assignment.expr);
-  var_types_.emplace(assignment.var->ident, type_);
+  var_types_.emplace(assignment.var->ident, sized_type_);
 }
 
 bool FieldAnalyser::compare_args(const ProbeArgs &args1, const ProbeArgs &args2)
 {
-  auto pred = [](auto a, auto b) { return a.first == b.first; };
+  using ProbeArgsValue = ProbeArgs::value_type;
+  auto pred = [](const ProbeArgsValue &a, const ProbeArgsValue &b) {
+    return a.first == b.first;
+  };
 
   return args1.size() == args2.size() &&
          std::equal(args1.begin(), args1.end(), args2.begin(), pred);
@@ -171,9 +182,6 @@ bool FieldAnalyser::resolve_args(Probe &probe)
     if (probe_type != ProbeType::kfunc && probe_type != ProbeType::kretfunc &&
         probe_type != ProbeType::uprobe)
       continue;
-    // BEGIN and END are special uprobe cases that do not have args
-    if (ap->provider == "BEGIN" || ap->provider == "END")
-      continue;
 
     if (ap->need_expansion)
     {
@@ -197,6 +205,10 @@ bool FieldAnalyser::resolve_args(Probe &probe)
       for (auto &match : matches)
       {
         ProbeArgs args;
+        // Both uprobes and kfuncs have a target (binary for uprobes, kernel
+        // module for kfuncs).
+        std::string func = match;
+        std::string target = erase_prefix(func);
 
         // Trying to attach to multiple kfuncs. If some of them fails on
         // argument resolution, do not fail hard, just print a warning and
@@ -205,9 +217,9 @@ bool FieldAnalyser::resolve_args(Probe &probe)
         {
           try
           {
-            bpftrace_.btf_.resolve_args(match,
-                                        first ? ap_args_ : args,
-                                        probe_type == ProbeType::kretfunc);
+            bpftrace_.btf_->resolve_args(func,
+                                         first ? ap_args_ : args,
+                                         probe_type == ProbeType::kretfunc);
           }
           catch (const std::runtime_error &e)
           {
@@ -217,10 +229,7 @@ bool FieldAnalyser::resolve_args(Probe &probe)
         }
         else // uprobe
         {
-          std::string func = match;
-          std::string file = erase_prefix(func);
-
-          Dwarf *dwarf = bpftrace_.get_dwarf(file);
+          Dwarf *dwarf = bpftrace_.get_dwarf(target);
           if (dwarf)
           {
             args = dwarf->resolve_args(func);
@@ -228,7 +237,7 @@ bool FieldAnalyser::resolve_args(Probe &probe)
               ap_args_ = args;
           }
           else
-            LOG(WARNING, ap->loc, err_) << "No debuginfo found for " << file;
+            LOG(WARNING, ap->loc, err_) << "No debuginfo found for " << target;
         }
 
         if (!first && !compare_args(args, ap_args_))
@@ -248,9 +257,9 @@ bool FieldAnalyser::resolve_args(Probe &probe)
       {
         try
         {
-          bpftrace_.btf_.resolve_args(ap->func,
-                                      ap_args_,
-                                      probe_type == ProbeType::kretfunc);
+          bpftrace_.btf_->resolve_args(ap->func,
+                                       ap_args_,
+                                       probe_type == ProbeType::kretfunc);
         }
         catch (const std::runtime_error &e)
         {
@@ -291,6 +300,19 @@ bool FieldAnalyser::resolve_args(Probe &probe)
   return true;
 }
 
+void FieldAnalyser::resolve_fields(SizedType &type)
+{
+  if (!type.IsRecordTy())
+    return;
+
+  for (auto &ap : *probe_->attach_points)
+    if (Dwarf *dwarf = bpftrace_.get_dwarf(*ap))
+      dwarf->resolve_fields(type);
+
+  if (type.GetFieldCount() == 0 && bpftrace_.has_btf_data())
+    bpftrace_.btf_->resolve_fields(type);
+}
+
 void FieldAnalyser::visit(Probe &probe)
 {
   probe_ = &probe;
