@@ -1,11 +1,16 @@
 #pragma once
 
-#include "bpftrace.h"
-#include "visitors.h"
 #include <iostream>
 #include <string>
 #include <unordered_set>
 
+#include "ast/visitors.h"
+#include "bpftrace.h"
+
+namespace libbpf {
+#include "libbpf/bpf.h"
+} // namespace libbpf
+
 namespace bpftrace {
 namespace ast {
 
@@ -16,9 +21,8 @@ public:
                          BPFtrace &bpftrace,
                          std::ostream &out = std::cerr)
       : root_(root),
-        type_(""),
         bpftrace_(bpftrace),
-        prog_type_(BPF_PROG_TYPE_UNSPEC),
+        prog_type_(libbpf::BPF_PROG_TYPE_UNSPEC),
         out_(out)
   { }
 
@@ -37,13 +41,14 @@ public:
 private:
   bool resolve_args(Probe &probe);
   bool compare_args(const ProbeArgs &args1, const ProbeArgs &args2);
+  void resolve_fields(SizedType &type);
 
   Node *root_;
   ProbeType probe_type_;
   std::string attach_func_;
-  std::string    type_;
+  SizedType sized_type_;
   BPFtrace      &bpftrace_;
-  bpf_prog_type  prog_type_;
+  libbpf::bpf_prog_type prog_type_;
   bool           has_builtin_args_;
   Probe         *probe_;
 
@@ -51,7 +56,7 @@ private:
   std::ostringstream  err_;
 
   ProbeArgs ap_args_;
-  std::map<std::string, std::string> var_types_;
+  std::map<std::string, SizedType> var_types_;
 };
 
 } // namespace ast
