@@ -18,6 +18,7 @@
 #include "map.h"
 #include "mapmanager.h"
 #include "output.h"
+#include "pcap_writer.h"
 #include "printf.h"
 #include "probe_matcher.h"
 #include "procmon.h"
@@ -42,6 +43,7 @@ enum class DebugLevel;
 extern DebugLevel bt_debug;
 extern bool bt_quiet;
 extern bool bt_verbose;
+extern bool bt_verbose2;
 
 enum class DebugLevel
 {
@@ -94,7 +96,6 @@ public:
         out_(std::move(o)),
         feature_(std::make_unique<BPFfeature>()),
         probe_matcher_(std::make_unique<ProbeMatcher>(this)),
-        btf_(this),
         ncpus_(get_possible_cpus().size())
   {
   }
@@ -126,6 +127,8 @@ public:
                             struct symbol *sym,
                             const std::string &path) const;
   std::string resolve_mac_address(const uint8_t *mac_addr) const;
+  std::string resolve_cgroup_path(uint64_t cgroup_path_id,
+                                  uint64_t cgroup_id) const;
   virtual std::string extract_func_symbols_from_path(const std::string &path) const;
   std::string resolve_probe(uint64_t probe_id) const;
   uint64_t resolve_cgroupid(const std::string &path) const;
@@ -139,15 +142,22 @@ public:
   std::optional<long> get_int_literal(const ast::Expression *expr) const;
   std::optional<std::string> get_watchpoint_binary_path() const;
   virtual bool is_traceable_func(const std::string &func_name) const;
+  std::unordered_set<std::string> get_func_modules(
+      const std::string &func_name) const;
+  int create_pcaps(void);
+  void close_pcaps(void);
+  bool write_pcaps(uint64_t id, uint64_t ns, uint8_t *pkt, unsigned int size);
 
+  void parse_btf(const std::set<std::string> &modules);
+  bool has_btf_data() const;
   Dwarf *get_dwarf(const std::string &filename);
   Dwarf *get_dwarf(const ast::AttachPoint &attachpoint);
 
-  std::vector<std::unique_ptr<AttachedProbe>> attached_probes_;
   std::string cmd_;
   bool finalize_ = false;
-  // Global variable checking if an exit signal was received
+  // Global variables checking if an exit/usr1 signal was received
   static volatile sig_atomic_t exitsig_recv;
+  static volatile sig_atomic_t sigusr1_recv;
 
   RequiredResources resources;
   MapManager maps;
@@ -155,7 +165,12 @@ public:
   StructManager structs;
   std::map<std::string, std::string> macros_;
   std::map<std::string, uint64_t> enums_;
-  std::unordered_set<std::string> traceable_funcs_;
+  std::map<libbpf::bpf_func_id, location> helper_use_loc_;
+  // mapping traceable functions to modules (or "vmlinux") that they appear in
+  FuncsModulesMap traceable_funcs_;
+  std::vector<std::unique_ptr<AttachedProbe>> attached_probes_;
+
+  std::map<std::string, std::unique_ptr<PCAPwriter>> pcap_writers;
 
   unsigned int join_argnum_ = 16;
   unsigned int join_argsize_ = 1024;
@@ -166,6 +181,7 @@ public:
   uint64_t mapmax_ = 4096;
   size_t cat_bytes_max_ = 10240;
   uint64_t max_probes_ = 512;
+  uint64_t max_programs_ = 512;
   uint64_t log_size_ = 1000000;
   uint64_t perf_rb_pages_ = 64;
   uint64_t max_type_res_iterations = 0;
@@ -186,7 +202,7 @@ public:
 
   std::unique_ptr<ProbeMatcher> probe_matcher_;
 
-  BTF btf_;
+  std::unique_ptr<BTF> btf_;
   std::unordered_set<std::string> btf_set_;
   std::map<std::string, ProbeArgs> ap_args_;
   std::unique_ptr<ChildProcBase> child_;
@@ -214,12 +230,13 @@ private:
       int pid,
       bool file_activation);
   int setup_perf_events();
-  void poll_perf_events(int epollfd, bool drain = false);
+  void poll_perf_events(bool drain = false);
   int print_map_hist(IMap &map, uint32_t top, uint32_t div);
   int print_map_stats(IMap &map, uint32_t top, uint32_t div);
   static uint64_t read_address_from_output(std::string output);
   std::vector<uint8_t> find_empty_key(IMap &map, size_t size) const;
   bool has_iter_ = false;
+  int epollfd_ = -1;
 
   std::unordered_map<std::string, std::unique_ptr<Dwarf>> dwarves_;
 };
