@@ -40,6 +40,11 @@ const std::vector<CXUnsavedFile> &getDefaultHeaders()
         .Contents = stdarg_h,
         .Length = stdarg_h_len,
     },
+    {
+        .Filename = "/bpftrace/include/stdbool.h",
+        .Contents = stdbool_h,
+        .Length = stdbool_h_len,
+    },
     {
         .Filename = "/bpftrace/include/stddef.h",
         .Contents = stddef_h,
@@ -565,7 +570,14 @@ bool ClangParser::visit_children(CXCursor &cursor, BPFtrace &bpftrace)
 
           // Initialize a new record type if needed
           if (!structs.Has(ptypestr))
-            structs.Add(ptypestr, ptypesize);
+            structs.Add(ptypestr, ptypesize, false);
+
+          auto str = structs.Lookup(ptypestr).lock();
+          if (str->allow_override)
+          {
+            str->ClearFields();
+            str->allow_override = false;
+          }
 
           // No need to worry about redefined types b/c we should have already
           // checked clang diagnostics. The diagnostics will tell us if we have
@@ -707,6 +719,9 @@ bool ClangParser::parse(ast::Program *program, BPFtrace &bpftrace, std::vector<s
   StderrSilencer silencer;
   silencer.silence();
 #endif
+  if (program->c_definitions.empty() && bpftrace.btf_set_.empty())
+    return true;
+
   input = "#include <__btf_generated_header.h>\n" + program->c_definitions;
 
   input_files = getTranslationUnitFiles(CXUnsavedFile{
@@ -715,13 +730,17 @@ bool ClangParser::parse(ast::Program *program, BPFtrace &bpftrace, std::vector<s
       .Length = input.size(),
   });
 
-  // clang-format off
-  args = {
-    "-isystem", "/usr/local/include",
-    "-isystem", "/bpftrace/include",
-    "-isystem", "/usr/include",
-  };
-  // clang-format on
+  args = { "-isystem", "/bpftrace/include" };
+  auto system_paths = system_include_paths();
+  for (auto &path : system_paths)
+  {
+    args.push_back("-isystem");
+    args.push_back(path.c_str());
+  }
+  std::string arch_path = get_arch_include_path();
+  args.push_back("-isystem");
+  args.push_back(arch_path.c_str());
+
   for (auto &flag : extra_flags)
   {
     args.push_back(flag.c_str());
@@ -731,13 +750,13 @@ bool ClangParser::parse(ast::Program *program, BPFtrace &bpftrace, std::vector<s
   // The header must be the last file in the vector since the following methods
   // count on it.
   // If BTF is not available, the header is empty.
-  input_files.emplace_back(bpftrace.btf_.has_data()
+  input_files.emplace_back(bpftrace.has_btf_data()
                                ? get_btf_generated_header(bpftrace)
                                : get_empty_btf_generated_header());
 
   bool btf_conflict = false;
   ClangParserHandler handler;
-  if (bpftrace.btf_.has_data())
+  if (bpftrace.has_btf_data())
   {
     // We set these args early because some systems may not have <linux/types.h>
     // (containers) and fully rely on BTF.
@@ -748,6 +767,9 @@ bool ClangParser::parse(ast::Program *program, BPFtrace &bpftrace, std::vector<s
     // Since we're omitting <linux/types.h> there's no reason to
     // add the wokarounds for it
     args.push_back("-D__CLANG_WORKAROUNDS_H");
+    // Let script know we have BTF -- this is useful for prewritten tools to
+    // conditionally include headers if BTF isn't available.
+    args.push_back("-DBPFTRACE_HAVE_BTF");
 
     if (handler.parse_file("definitions.h", input, args, input_files, false) &&
         handler.has_redefinition_error())
@@ -780,6 +802,7 @@ bool ClangParser::parse(ast::Program *program, BPFtrace &bpftrace, std::vector<s
     // taken from BTF. We cannot use BTF in such a case.
     args.pop_back();
     args.pop_back();
+    args.pop_back();
     input_files.back() = get_empty_btf_generated_header();
   }
 
@@ -864,7 +887,7 @@ void ClangParser::resolve_unknown_typedefs_from_btf(BPFtrace &bpftrace)
 
 CXUnsavedFile ClangParser::get_btf_generated_header(BPFtrace &bpftrace)
 {
-  btf_cdef = bpftrace.btf_.c_def(bpftrace.btf_set_);
+  btf_cdef = bpftrace.btf_->c_def(bpftrace.btf_set_);
   return CXUnsavedFile{
     .Filename = "/bpftrace/include/__btf_generated_header.h",
     .Contents = btf_cdef.c_str(),
@@ -882,4 +905,38 @@ CXUnsavedFile ClangParser::get_empty_btf_generated_header()
   };
 }
 
+std::string ClangParser::get_arch_include_path()
+{
+  struct utsname utsname;
+  uname(&utsname);
+  return "/usr/include/" + std::string(utsname.machine) + "-linux-gnu";
+}
+
+std::vector<std::string> ClangParser::system_include_paths()
+{
+  std::vector<std::string> result;
+  try
+  {
+    auto clang = "clang-" + std::to_string(LLVM_VERSION_MAJOR);
+    auto cmd = clang + " -Wp,-v -x c -fsyntax-only /dev/null 2>&1";
+    auto check = exec_system(cmd.c_str());
+    std::istringstream lines(check);
+    std::string line;
+    while (std::getline(lines, line) &&
+           line != "#include <...> search starts here:")
+    {
+    }
+    while (std::getline(lines, line) && line != "End of search list.")
+      result.push_back(trim(line));
+  }
+  catch (std::runtime_error &)
+  { // If exec_system fails, just ignore it
+  }
+
+  if (result.empty())
+    result = { "/usr/local/include", "/usr/include" };
+
+  return result;
+}
+
 } // namespace bpftrace
