@@ -13,7 +13,7 @@ public:
           int max_entries)
       : FakeMap(name, type, key, 0, 0, 0, max_entries){};
   FakeMap(const SizedType &type);
-  FakeMap(enum bpf_map_type map_type);
+  FakeMap(libbpf::bpf_map_type map_type);
   FakeMap(const std::string &name,
           const SizedType &type,
           const MapKey &key,
@@ -22,7 +22,7 @@ public:
           int step,
           int max_entries);
   FakeMap(const std::string &name,
-          enum bpf_map_type type,
+          libbpf::bpf_map_type type,
           int key_size,
           int value_size,
           int max_entries,
