@@ -5,7 +5,8 @@
 #include "bpftrace.h"
 #include "log.h"
 #include "utils.h"
-#include <bcc/libbpf.h>
+
+#include <bpf/bpf.h>
 
 #include "map.h"
 #include "mapmanager.h"
@@ -14,12 +15,12 @@ namespace bpftrace {
 
 namespace {
 
-int create_map(enum bpf_map_type map_type,
+int create_map(libbpf::bpf_map_type map_type,
                const std::string &name,
                int key_size,
                int value_size,
                int max_entries,
-               int flags)
+               unsigned int flags)
 {
   std::string fixed_name;
   const std::string *name_ptr = &name;
@@ -28,13 +29,15 @@ int create_map(enum bpf_map_type map_type,
     fixed_name = "AT_" + name.substr(1);
     name_ptr = &fixed_name;
   }
-#ifdef HAVE_BCC_CREATE_MAP
-  return bcc_create_map(
-      map_type, name_ptr->c_str(), key_size, value_size, max_entries, flags);
-#else
-  return bpf_create_map(
-      map_type, name_ptr->c_str(), key_size, value_size, max_entries, flags);
-#endif
+
+  LIBBPF_OPTS(bpf_map_create_opts, opts);
+  opts.map_flags = flags;
+  return bpf_map_create(static_cast<::bpf_map_type>(map_type),
+                        name_ptr->c_str(),
+                        key_size,
+                        value_size,
+                        max_entries,
+                        &opts);
 }
 
 } // namespace
@@ -71,7 +74,7 @@ Map::Map(const std::string &name,
 }
 
 Map::Map(const std::string &name,
-         enum bpf_map_type type,
+         libbpf::bpf_map_type type,
          int key_size,
          int value_size,
          int max_entries,
@@ -107,12 +110,12 @@ Map::Map(const std::string &name,
 // bpf_get_stack() to write into. Using the stack directly wouldn't work well
 // b/c it would take too much stack space.
 //
-// The temporary fix is to bump the map size to 128K. Any futher bumps should
+// The temporary fix is to bump the map size to 128K. Any further bumps should
 // warrant consideration of the previous paragraph.
 Map::Map(const SizedType &type) : IMap(type)
 {
   int key_size = 4;
-  int value_size = sizeof(uintptr_t) * type.stack_type.limit;
+  int value_size = sizeof(uint64_t) * type.stack_type.limit;
   int max_entries = 128 << 10;
   int flags = 0;
 
@@ -120,17 +123,23 @@ Map::Map(const SizedType &type) : IMap(type)
       map_type_, "stack", key_size, value_size, max_entries, flags);
   if (mapfd_ < 0)
   {
-    LOG(ERROR)
-        << "failed to create stack id map\n"
-        // TODO (mmarchini): Check perf_event_max_stack in the semantic_analyzer
-        << "This might have happened because kernel.perf_event_max_stack "
-        << "is smaller than " << type.stack_type.limit
-        << ". Try to tweak this value with "
-        << "sysctl kernel.perf_event_max_stack=<new value>";
+    LOG(ERROR) << "failed to create stack id map: " << strerror(errno);
+    if (errno == ENOMEM)
+      LOG(ERROR) << "Tried to allocate " << max_entries << " entries of size "
+                 << value_size
+                 << ". Consider limiting the number of stack frames captured.";
+    else
+      LOG(ERROR)
+          // TODO (mmarchini): Check perf_event_max_stack in the
+          // semantic_analyzer
+          << "This might have happened because kernel.perf_event_max_stack "
+          << "is smaller than " << type.stack_type.limit
+          << ". Try to tweak this value with "
+          << "sysctl kernel.perf_event_max_stack=<new value>";
   }
 }
 
-Map::Map(enum bpf_map_type map_type) : IMap(map_type)
+Map::Map(libbpf::bpf_map_type map_type) : IMap(map_type)
 {
   std::vector<int> cpus = get_online_cpus();
   int key_size = 4;
@@ -251,8 +260,8 @@ std::string to_string(MapManager::Type t)
       return "join";
     case MapManager::Type::Elapsed:
       return "elapsed";
-    case MapManager::Type::SeqPrintfData:
-      return "seq_printf_data";
+    case MapManager::Type::MappedPrintfData:
+      return "mapped_printf_data";
   }
   return {}; // unreached
 }
