@@ -20,13 +20,13 @@ public:
       int step,
       int max_entries);
   Map(const std::string &name,
-      enum bpf_map_type type,
+      libbpf::bpf_map_type type,
       int key_size,
       int value_size,
       int max_entries,
       int flags);
   Map(const SizedType &type);
-  Map(enum bpf_map_type map_type);
+  Map(libbpf::bpf_map_type map_type);
   virtual ~Map() override;
 };
 } // namespace bpftrace
