@@ -275,6 +275,13 @@ std::string Output::value_to_str(BPFtrace &bpftrace,
             ->nsecs_since_boot);
   else if (type.IsMacAddressTy())
     return bpftrace.resolve_mac_address(value.data());
+  else if (type.IsCgroupPathTy())
+    return bpftrace.resolve_cgroup_path(
+        reinterpret_cast<AsyncEvent::CgroupPath *>(value.data())
+            ->cgroup_path_id,
+        reinterpret_cast<AsyncEvent::CgroupPath *>(value.data())->cgroup_id);
+  else if (type.IsStrerrorTy())
+    return strerror(read_data<uint64_t>(value.data()));
   else
     return std::to_string(read_data<int64_t>(value.data()) / div);
 }
@@ -537,6 +544,19 @@ void TextOutput::attached_probes(uint64_t num_probes) const
     out_ << "Attaching " << num_probes << " probes..." << std::endl;
 }
 
+void TextOutput::helper_error(const std::string &helper,
+                              int retcode,
+                              const location &loc) const
+{
+  std::stringstream msg;
+  msg << "Failed to " << helper << ": ";
+  if (retcode < 0)
+    msg << strerror(-retcode) << " (" << retcode << ")";
+  else
+    msg << retcode;
+  LOG(WARNING, loc, out_) << msg.str();
+}
+
 std::string TextOutput::field_to_str(const std::string &name,
                                      const std::string &value) const
 {
@@ -614,9 +634,13 @@ std::string JsonOutput::json_escape(const std::string &str) const
         break;
 
       default:
-        if ('\x00' <= c && c <= '\x1f') {
+        // c always >= '\x00'
+        if (c <= '\x1f')
+        {
           escaped << "\\u" << std::hex << std::setw(4) << std::setfill('0') << (int)c;
-        } else {
+        }
+        else
+        {
           escaped << c;
         }
     }
@@ -653,7 +677,7 @@ std::string JsonOutput::hist_to_str(const std::vector<uint64_t> &values,
   int min_index, max_index, max_value;
   hist_prepare(values, min_index, max_index, max_value);
   if (max_index == -1)
-    return "";
+    return "[]";
 
   std::ostringstream res;
   res << "[";
@@ -697,7 +721,7 @@ std::string JsonOutput::lhist_to_str(const std::vector<uint64_t> &values,
   int max_index, max_value, buckets, start_value, end_value;
   lhist_prepare(values, min, max, step, max_index, max_value, buckets, start_value, end_value);
   if (max_index == -1)
-    return "";
+    return "[]";
 
   std::ostringstream res;
   res << "[";
@@ -804,6 +828,15 @@ void JsonOutput::attached_probes(uint64_t num_probes) const
   message(MessageType::attached_probes, "probes", num_probes);
 }
 
+void JsonOutput::helper_error(const std::string &helper,
+                              int retcode,
+                              const location &loc) const
+{
+  out_ << "{\"type\": \"helper_error\", \"helper\": \"" << helper
+       << "\", \"retcode\": " << retcode << ", \"line\": " << loc.begin.line
+       << ", \"col\": " << loc.begin.column << "}" << std::endl;
+}
+
 std::string JsonOutput::field_to_str(const std::string &name,
                                      const std::string &value) const
 {
