@@ -1,10 +1,12 @@
 #include "relocator.h"
 
-#include <bcc/libbpf.h>
-
 #include "bpftrace.h"
 #include "log.h"
 
+namespace libbpf {
+#include "libbpf/bpf.h"
+} // namespace libbpf
+
 namespace bpftrace {
 
 Relocator::Relocator(std::tuple<uint8_t *, uintptr_t> func, BPFtrace &bpftrace)
