@@ -121,7 +121,7 @@ bool SizedType::IsByteArray() const
 {
   return type == Type::string || type == Type::usym || type == Type::inet ||
          type == Type::buffer || type == Type::timestamp ||
-         type == Type::mac_address;
+         type == Type::mac_address || type == Type::cgroup_path;
 }
 
 bool SizedType::IsAggregate() const
@@ -183,6 +183,8 @@ std::string typestr(Type t)
     case Type::tuple:    return "tuple";    break;
     case Type::timestamp:return "timestamp";break;
     case Type::mac_address: return "mac_address"; break;
+    case Type::cgroup_path: return "cgroup_path"; break;
+    case Type::strerror: return "strerror"; break;
       // clang-format on
   }
 
@@ -223,9 +225,11 @@ std::string probetypeName(const std::string &probeName)
 
 std::string probetypeName(ProbeType t)
 {
-   switch (t)
+  // clang-format off
+  switch (t)
   {
     case ProbeType::invalid:     return "invalid";     break;
+    case ProbeType::special:     return "special";     break;
     case ProbeType::kprobe:      return "kprobe";      break;
     case ProbeType::kretprobe:   return "kretprobe";   break;
     case ProbeType::uprobe:      return "uprobe";      break;
@@ -237,15 +241,12 @@ std::string probetypeName(ProbeType t)
     case ProbeType::software:    return "software";    break;
     case ProbeType::hardware:    return "hardware";    break;
     case ProbeType::watchpoint:  return "watchpoint";  break;
-    case ProbeType::asyncwatchpoint:
-      return "asyncwatchpoint";
-      break;
+    case ProbeType::asyncwatchpoint: return "asyncwatchpoint"; break;
     case ProbeType::kfunc:       return "kfunc";       break;
     case ProbeType::kretfunc:    return "kretfunc";    break;
-    case ProbeType::iter:
-      return "iter";
-      break;
+    case ProbeType::iter:        return "iter";        break;
   }
+  // clang-format on
 
   return {}; // unreached
 }
@@ -469,6 +470,16 @@ SizedType CreateMacAddress()
   return st;
 }
 
+SizedType CreateCgroupPath()
+{
+  return SizedType(Type::cgroup_path, 16);
+}
+
+SizedType CreateStrerror()
+{
+  return SizedType(Type::strerror, 8);
+}
+
 bool SizedType::IsSigned(void) const
 {
   return is_signed_;
@@ -540,6 +551,29 @@ std::weak_ptr<const Struct> SizedType::GetStruct() const
   return inner_struct_;
 }
 
+// Checks if values of this type can be copied into values of another type
+// Currently checks if strings in the other type (at corresponding places) are
+// larger.
+bool SizedType::FitsInto(const SizedType &t) const
+{
+  if (IsStringTy() && t.IsStringTy())
+    return GetSize() <= t.GetSize();
+
+  if (IsTupleTy() && t.IsTupleTy())
+  {
+    if (GetFieldCount() != t.GetFieldCount())
+      return false;
+
+    for (ssize_t i = 0; i < GetFieldCount(); i++)
+    {
+      if (!GetField(i).type.FitsInto(t.GetField(i).type))
+        return false;
+    }
+    return true;
+  }
+  return IsEqual(t);
+}
+
 } // namespace bpftrace
 
 namespace std {
@@ -592,6 +626,8 @@ size_t hash<bpftrace::SizedType>::operator()(
     case bpftrace::Type::buffer:
     case bpftrace::Type::timestamp:
     case bpftrace::Type::mac_address:
+    case bpftrace::Type::cgroup_path:
+    case bpftrace::Type::strerror:
       break;
   }
 
