@@ -47,7 +47,9 @@ enum class Type
   buffer,
   tuple,
   timestamp,
-  mac_address
+  mac_address,
+  cgroup_path,
+  strerror
   // clang-format on
 };
 
@@ -85,7 +87,6 @@ private:
   }
 };
 
-class BPFtrace;
 struct Struct;
 struct Field;
 
@@ -195,6 +196,7 @@ public:
   bool operator==(const SizedType &t) const;
   bool operator!=(const SizedType &t) const;
   bool IsSameType(const SizedType &t) const;
+  bool FitsInto(const SizedType &t) const;
 
   bool IsPrintableTy()
   {
@@ -362,6 +364,14 @@ public:
   {
     return type == Type::mac_address;
   };
+  bool IsCgroupPathTy(void) const
+  {
+    return type == Type::cgroup_path;
+  };
+  bool IsStrerrorTy(void) const
+  {
+    return type == Type::strerror;
+  };
 
   friend std::ostream &operator<<(std::ostream &, const SizedType &);
   friend std::ostream &operator<<(std::ostream &, Type);
@@ -420,12 +430,15 @@ SizedType CreateKSym();
 SizedType CreateBuffer(size_t size);
 SizedType CreateTimestamp();
 SizedType CreateMacAddress();
+SizedType CreateCgroupPath();
+SizedType CreateStrerror();
 
 std::ostream &operator<<(std::ostream &os, const SizedType &type);
 
 enum class ProbeType
 {
   invalid,
+  special,
   kprobe,
   kretprobe,
   uprobe,
@@ -458,8 +471,8 @@ const std::vector<ProbeItem> PROBE_LIST = {
   { "uprobe", "u", ProbeType::uprobe },
   { "uretprobe", "ur", ProbeType::uretprobe },
   { "usdt", "U", ProbeType::usdt },
-  { "BEGIN", "BEGIN", ProbeType::uprobe },
-  { "END", "END", ProbeType::uprobe },
+  { "BEGIN", "BEGIN", ProbeType::special },
+  { "END", "END", ProbeType::special },
   { "tracepoint", "t", ProbeType::tracepoint },
   { "profile", "p", ProbeType::profile },
   { "interval", "i", ProbeType::interval },
@@ -487,6 +500,7 @@ struct Probe
   std::string orig_name;        // original full probe name,
                                 // before wildcard expansion
   std::string name;             // full probe name
+  bool need_expansion;
   std::string pin;              // pin file for iterator probes
   std::string ns;               // for USDT probes, if provider namespace not from path
   uint64_t loc = 0;             // for USDT probes
@@ -500,6 +514,7 @@ struct Probe
   bool async = false; // for watchpoint probes, if it's an async watchpoint
   uint64_t address = 0;
   uint64_t func_offset = 0;
+  std::vector<std::string> funcs;
 
 private:
   friend class cereal::access;
@@ -523,7 +538,8 @@ private:
             mode,
             async,
             address,
-            func_offset);
+            func_offset,
+            funcs);
   }
 };
 
@@ -548,6 +564,7 @@ enum class AsyncAction
   strftime,
   watchpoint_attach,
   watchpoint_detach,
+  skboutput,
   // clang-format on
 };
 
