@@ -1,5 +1,6 @@
 #pragma once
 
+#include <cstdint>
 #include <optional>
 #include <string>
 #include <vector>
@@ -9,6 +10,7 @@ struct usdt_probe_entry
   std::string path;
   std::string provider;
   std::string name;
+  uint64_t semaphore_offset;
   int num_locations;
 };
 
