@@ -1,5 +1,6 @@
 #pragma once
 
+#include <cstdint>
 #include <cstring>
 #include <exception>
 #include <iostream>
@@ -8,6 +9,7 @@
 #include <string>
 #include <sys/utsname.h>
 #include <tuple>
+#include <unordered_map>
 #include <unordered_set>
 #include <vector>
 
@@ -125,6 +127,9 @@ struct DeprecatedName
   bool show_warning = true;
 };
 
+typedef std::unordered_map<std::string, std::unordered_set<std::string>>
+    FuncsModulesMap;
+
 static std::vector<DeprecatedName> DEPRECATED_LIST =
 {
 };
@@ -138,6 +143,7 @@ static std::vector<std::string> UNSAFE_BUILTIN_FUNCS = {
 static std::vector<std::string> COMPILE_TIME_FUNCS = { "cgroupid" };
 
 bool get_uint64_env_var(const ::std::string &str, uint64_t &dest);
+bool get_bool_env_var(const ::std::string &str, bool &dest, bool neg = false);
 std::string get_pid_exe(pid_t pid);
 std::string get_pid_exe(const std::string &pid);
 bool has_wildcard(const std::string &str);
@@ -149,6 +155,9 @@ bool wildcard_match(const std::string &str,
                     std::vector<std::string> &tokens,
                     bool start_wildcard,
                     bool end_wildcard);
+std::vector<std::string> get_wildcard_tokens(const std::string &input,
+                                             bool &start_wildcard,
+                                             bool &end_wildcard);
 std::vector<int> get_online_cpus();
 std::vector<int> get_possible_cpus();
 bool is_dir(const std::string &path);
@@ -158,7 +167,13 @@ std::tuple<std::string, std::string> get_kernel_dirs(
 std::vector<std::string> get_kernel_cflags(const char *uname_machine,
                                            const std::string &ksrc,
                                            const std::string &kobj);
-std::unordered_set<std::string> get_traceable_funcs();
+std::string get_cgroup_path_in_hierarchy(uint64_t cgroupid,
+                                         std::string base_path);
+std::vector<std::pair<std::string, std::string>> get_cgroup_hierarchy_roots();
+std::vector<std::pair<std::string, std::string>> get_cgroup_paths(
+    uint64_t cgroupid,
+    std::string filter);
+FuncsModulesMap get_traceable_funcs();
 const std::string &is_deprecated(const std::string &str);
 bool is_unsafe_func(const std::string &func_name);
 bool is_compile_time_func(const std::string &func_name);
@@ -172,10 +187,15 @@ std::string str_join(const std::vector<std::string> &list,
 bool is_numeric(const std::string &str);
 bool symbol_has_cpp_mangled_signature(const std::string &sym_name);
 pid_t parse_pid(const std::string &str);
-std::string hex_format_buffer(const char *buf, size_t size);
+std::string hex_format_buffer(const char *buf,
+                              size_t size,
+                              bool keep_ascii = true,
+                              bool escape_hex = true);
 std::optional<std::string> abs_path(const std::string &rel_path);
 bool symbol_has_module(const std::string &symbol);
 std::string strip_symbol_module(const std::string &symbol);
+std::pair<std::string, std::string> split_symbol_module(
+    const std::string &symbol);
 
 // Generate object file section name for a given probe
 inline std::string get_section_name_for_probe(
@@ -259,4 +279,7 @@ inline void hash_combine(std::size_t &seed, const T &value)
   seed ^= hasher(value) + 0x9e3779b9 + (seed << 6) + (seed >> 2);
 }
 
+// Returns the width in bits of kernel pointers.
+int get_kernel_ptr_width();
+
 } // namespace bpftrace
