@@ -10,6 +10,8 @@ namespace bpftrace {
 namespace test {
 namespace clang_parser {
 
+#include "btf_common.h"
+
 static void parse(const std::string &input, BPFtrace &bpftrace, bool result = true,
                   const std::string& probe = "kprobe:sys_read { 1 }")
 {
@@ -17,11 +19,11 @@ static void parse(const std::string &input, BPFtrace &bpftrace, bool result = tr
   Driver driver(bpftrace);
   ASSERT_EQ(driver.parse_str(extended_input), 0);
 
-  ast::FieldAnalyser fields(driver.root_, bpftrace);
+  ast::FieldAnalyser fields(driver.root.get(), bpftrace);
   EXPECT_EQ(fields.analyse(), 0);
 
   ClangParser clang;
-  ASSERT_EQ(clang.parse(driver.root_, bpftrace), result);
+  ASSERT_EQ(clang.parse(driver.root.get(), bpftrace), result);
 }
 
 TEST(clang_parser, integers)
@@ -202,8 +204,13 @@ TEST(clang_parser, nested_struct_no_type)
   // since they are called bar and baz
   parse("struct Foo { struct { int x; } bar; union { int y; } baz; }", bpftrace);
 
+#if LLVM_VERSION_MAJOR >= 13
+  std::string bar_name = "struct Foo::(unnamed at definitions.h:2:14)";
+  std::string baz_name = "union Foo::(unnamed at definitions.h:2:37)";
+#else
   std::string bar_name = "struct Foo::(anonymous at definitions.h:2:14)";
   std::string baz_name = "union Foo::(anonymous at definitions.h:2:37)";
+#endif
 
   ASSERT_TRUE(bpftrace.structs.Has("struct Foo"));
   ASSERT_TRUE(bpftrace.structs.Has(bar_name));
@@ -511,10 +518,6 @@ TEST(clang_parser, parse_fail)
   parse("struct a { int a; struct b b; };", bpftrace, false);
 }
 
-#ifdef HAVE_LIBBPF_BTF_DUMP
-
-#include "btf_common.h"
-
 class clang_parser_btf : public test_btf
 {
 };
@@ -522,12 +525,14 @@ class clang_parser_btf : public test_btf
 TEST_F(clang_parser_btf, btf)
 {
   BPFtrace bpftrace;
-  parse("", bpftrace, true,
-        "kprobe:sys_read {\n"
-        "  @x1 = (struct Foo1 *) curtask;\n"
-        "  @x2 = (struct Foo2 *) curtask;\n"
-        "  @x3 = (struct Foo3 *) curtask;\n"
-        "}");
+  bpftrace.parse_btf({});
+  parse("struct Foo { "
+        "  struct Foo1 f1;"
+        "  struct Foo2 f2;"
+        "  struct Foo3 f3;"
+        "  struct task_struct t;"
+        "}",
+        bpftrace);
 
   ASSERT_TRUE(bpftrace.structs.Has("struct Foo1"));
   ASSERT_TRUE(bpftrace.structs.Has("struct Foo2"));
@@ -595,51 +600,13 @@ TEST_F(clang_parser_btf, btf)
   EXPECT_EQ(foo2_field.offset, 8);
 }
 
-TEST_F(clang_parser_btf, btf_field_struct)
-{
-  BPFtrace bpftrace;
-  parse("",
-        bpftrace,
-        true,
-        "kprobe:sys_read {\n"
-        "  @x3 = ((struct Foo3 *) curtask)->foo2->g;\n"
-        "}");
-
-  /* task_struct->Foo3->Foo2->char */
-  EXPECT_EQ(bpftrace.btf_set_.size(), 4U);
-  EXPECT_NE(bpftrace.btf_set_.find("struct task_struct"), bpftrace.btf_set_.end());
-  EXPECT_NE(bpftrace.btf_set_.find("struct Foo3"), bpftrace.btf_set_.end());
-  EXPECT_NE(bpftrace.btf_set_.find("struct Foo2"), bpftrace.btf_set_.end());
-  EXPECT_NE(bpftrace.btf_set_.find("char"), bpftrace.btf_set_.end());
-}
-
-TEST_F(clang_parser_btf, btf_variable_field_struct)
-{
-  BPFtrace bpftrace;
-  parse("",
-        bpftrace,
-        true,
-        "kprobe:sys_read {\n"
-        "  @x1 = ((struct Foo3 *) curtask);\n"
-        "  @x2 = ((struct Foo1 *) curtask);\n"
-        "  @x3 = @x1->foo2;\n"
-        "}");
-
-  EXPECT_EQ(bpftrace.btf_set_.size(), 4U);
-  EXPECT_NE(bpftrace.btf_set_.find("struct task_struct"),
-            bpftrace.btf_set_.end());
-  EXPECT_NE(bpftrace.btf_set_.find("struct Foo1"), bpftrace.btf_set_.end());
-  // struct Foo2 should be added by @x1->foo2
-  EXPECT_NE(bpftrace.btf_set_.find("struct Foo2"), bpftrace.btf_set_.end());
-  EXPECT_NE(bpftrace.btf_set_.find("struct Foo3"), bpftrace.btf_set_.end());
-}
-
 TEST(clang_parser, btf_unresolved_typedef)
 {
   // size_t is defined in stddef.h, but if we have BTF, it should be possible to
   // extract it from there
   BPFtrace bpftrace;
-  if (!bpftrace.btf_.has_data())
+  bpftrace.parse_btf({});
+  if (!bpftrace.has_btf_data())
     GTEST_SKIP();
 
   parse("struct Foo { size_t x; };", bpftrace);
@@ -660,6 +627,7 @@ TEST_F(clang_parser_btf, btf_type_override)
 {
   // It should be possible to override types from BTF, ...
   BPFtrace bpftrace;
+  bpftrace.parse_btf({});
   parse("struct Foo1 { int a; };\n",
         bpftrace,
         true,
@@ -686,7 +654,6 @@ TEST_F(clang_parser_btf, btf_type_override)
         false,
         "kprobe:sys_read { @x1 = ((struct Foo3 *)curtask); }");
 }
-#endif // HAVE_LIBBPF_BTF_DUMP
 
 TEST(clang_parser, struct_typedef)
 {
