@@ -33,7 +33,7 @@ void test(BPFtrace &bpftrace,
 
   std::ostringstream out;
   Printer printer(out);
-  printer.print(driver.root_);
+  printer.print(driver.root.get());
   EXPECT_EQ(output, out.str());
 }
 
@@ -53,6 +53,7 @@ TEST(Parser, builtin_variables)
   test("kprobe:f { gid }", "Program\n kprobe:f\n  builtin: gid\n");
   test("kprobe:f { nsecs }", "Program\n kprobe:f\n  builtin: nsecs\n");
   test("kprobe:f { elapsed }", "Program\n kprobe:f\n  builtin: elapsed\n");
+  test("kprobe:f { numaid }", "Program\n kprobe:f\n  builtin: numaid\n");
   test("kprobe:f { cpu }", "Program\n kprobe:f\n  builtin: cpu\n");
   test("kprobe:f { curtask }", "Program\n kprobe:f\n  builtin: curtask\n");
   test("kprobe:f { rand }", "Program\n kprobe:f\n  builtin: rand\n");
@@ -380,6 +381,23 @@ TEST(semantic_analyser, compound_variable_assignments)
        "    int: 1\n");
 }
 
+TEST(Parser, compound_variable_assignment_binary_expr)
+{
+  test("kprobe:f { $a = 0; $a += 2 - 1 }",
+       "Program\n"
+       " kprobe:f\n"
+       "  =\n"
+       "   variable: $a\n"
+       "   int: 0\n"
+       "  =\n"
+       "   variable: $a\n"
+       "   +\n"
+       "    variable: $a\n"
+       "    -\n"
+       "     int: 2\n"
+       "     int: 1\n");
+}
+
 TEST(Parser, compound_map_assignments)
 {
   test("kprobe:f { @a <<= 1 }",
@@ -464,6 +482,20 @@ TEST(Parser, compound_map_assignments)
        "    int: 1\n");
 }
 
+TEST(Parser, compound_map_assignment_binary_expr)
+{
+  test("kprobe:f { @a += 2 - 1 }",
+       "Program\n"
+       " kprobe:f\n"
+       "  =\n"
+       "   map: @a\n"
+       "   +\n"
+       "    map: @a\n"
+       "    -\n"
+       "     int: 2\n"
+       "     int: 1\n");
+}
+
 TEST(Parser, integer_sizes)
 {
   test("kprobe:do_nanosleep { $x = 0x12345678; }",
@@ -990,6 +1022,7 @@ TEST(Parser, uprobe)
 
   test_parse_failure("uprobe:f { 1 }");
   test_parse_failure("uprobe { 1 }");
+  test_parse_failure("uprobe:/my/program*:0x1234 { 1 }");
 }
 
 TEST(Parser, usdt)
@@ -1210,6 +1243,7 @@ TEST(Parser, wildcard_probetype)
        " uprobe:/bin/sh:*\n"
        " usdt:/bin/sh:*\n"
        "  int: 1\n");
+  test_parse_failure("iter:task* { }");
 }
 
 TEST(Parser, wildcard_attach_points)
