@@ -1,4 +1,3 @@
-#include "ast/bpforc/bpforc.h"
 
 #include "ast/passes/codegen_llvm.h"
 #include "ast/passes/field_analyser.h"
@@ -17,6 +16,8 @@ namespace bpftrace {
 namespace test {
 namespace probe {
 
+#include "btf_common.h"
+
 using bpftrace::ast::AttachPoint;
 using bpftrace::ast::AttachPointList;
 using bpftrace::ast::Probe;
@@ -28,23 +29,25 @@ void gen_bytecode(const std::string &input, std::stringstream &out)
 
   ASSERT_EQ(driver.parse_str(input), 0);
 
-  ast::FieldAnalyser fields(driver.root_, *bpftrace);
+  ast::FieldAnalyser fields(driver.root.get(), *bpftrace);
   EXPECT_EQ(fields.analyse(), 0);
 
   ClangParser clang;
-  clang.parse(driver.root_, *bpftrace);
+  clang.parse(driver.root.get(), *bpftrace);
 
   // Override to mockbpffeature.
   bpftrace->feature_ = std::make_unique<MockBPFfeature>(true);
-  ast::SemanticAnalyser semantics(driver.root_, *bpftrace);
+  ast::SemanticAnalyser semantics(driver.root.get(), *bpftrace);
   ASSERT_EQ(semantics.analyse(), 0);
 
-  ast::ResourceAnalyser resource_analyser(driver.root_);
-  auto resources = resource_analyser.analyse();
+  ast::ResourceAnalyser resource_analyser(driver.root.get());
+  auto resources_optional = resource_analyser.analyse();
+  ASSERT_TRUE(resources_optional.has_value());
+  auto resources = resources_optional.value();
   ASSERT_EQ(resources.create_maps(*bpftrace, true), 0);
   bpftrace->resources = resources;
 
-  ast::CodegenLLVM codegen(driver.root_, *bpftrace);
+  ast::CodegenLLVM codegen(driver.root.get(), *bpftrace);
   codegen.generate_ir();
   codegen.DumpIR(out);
 }
@@ -72,11 +75,6 @@ TEST(probe, short_name)
   compare_bytecode("interval:s:1 { 1 }", "i:s:1 { 1 }");
 }
 
-#ifdef HAVE_LIBBPF_BTF_DUMP
-#ifdef HAVE_BCC_KFUNC
-
-#include "btf_common.h"
-
 class probe_btf : public test_btf
 {
 };
@@ -89,9 +87,6 @@ TEST_F(probe_btf, short_name)
   compare_bytecode("iter:task_file { 1 }", "it:task_file { 1 }");
 }
 
-#endif // HAVE_BCC_KFUNC
-#endif // HAVE_LIBBPF_BTF_DUMP
-
 } // namespace probe
 } // namespace test
 } // namespace bpftrace
