@@ -28,12 +28,14 @@ TestStruct = namedtuple(
         'before',
         'after',
         'suite',
-        'kernel',
+        'kernel_min',
+        'kernel_max',
         'requirement',
         'env',
         'arch',
         'feature_requirement',
         'neg_feature_requirement',
+        'will_fail'
     ],
 )
 
@@ -101,12 +103,14 @@ class TestParser(object):
         timeout = ''
         before = ''
         after = ''
-        kernel = ''
-        requirement = ''
+        kernel_min = ''
+        kernel_max = ''
+        requirement = []
         env = {}
         arch = []
         feature_requirement = set()
         neg_feature_requirement = set()
+        will_fail = False
 
         for item in test:
             item_split = item.split()
@@ -128,9 +132,11 @@ class TestParser(object):
             elif item_name == 'AFTER':
                 after = line
             elif item_name == 'MIN_KERNEL':
-                kernel = line
+                kernel_min = line
+            elif item_name == 'MAX_KERNEL':
+                kernel_max = line
             elif item_name == 'REQUIRES':
-                requirement = line
+                requirement.append(line)
             elif item_name == 'ENV':
                 for e in line.split():
                     k, v = e.split('=')
@@ -141,6 +147,7 @@ class TestParser(object):
                 features = {
                     "loop",
                     "btf",
+                    "kfunc",
                     "probe_read_kernel",
                     "dpath",
                     "uprobe_refcount",
@@ -150,6 +157,8 @@ class TestParser(object):
                     "libpath_resolv",
                     "dwarf",
                     "aot",
+                    "kprobe_multi",
+                    "skboutput",
                 }
 
                 for f in line.split(" "):
@@ -162,6 +171,8 @@ class TestParser(object):
                 unknown = (feature_requirement | neg_feature_requirement) - features
                 if len(unknown) > 0:
                     raise UnknownFieldError('%s is invalid for REQUIRES_FEATURE. Suite: %s' % (','.join(unknown), test_suite))
+            elif item_name == "WILL_FAIL":
+                will_fail = True
             else:
                 raise UnknownFieldError('Field %s is unknown. Suite: %s' % (item_name, test_suite))
 
@@ -185,9 +196,11 @@ class TestParser(object):
             before,
             after,
             test_suite,
-            kernel,
+            kernel_min,
+            kernel_max,
             requirement,
             env,
             arch,
             feature_requirement,
-            neg_feature_requirement)
+            neg_feature_requirement,
+            will_fail)
