@@ -1,6 +1,7 @@
 #include "tracepoint_format_parser.h"
 #include "mocks.h"
 #include "gtest/gtest.h"
+#include <driver.h>
 
 using namespace testing;
 
@@ -90,10 +91,10 @@ TEST(tracepoint_format_parser, data_loc)
   std::string input = "	field:__data_loc char[] msg;	offset:8;	size:4;	signed:1;";
 
   std::string expected =
-    "struct _tracepoint_syscalls_sys_enter_read\n"
-    "{\n"
-    "  int data_loc_msg;\n"
-    "};\n";
+      "struct _tracepoint_syscalls_sys_enter_read\n"
+      "{\n"
+      "  __attribute__((annotate(\"tp_data_loc\"))) int msg;\n"
+      "};\n";
 
   std::istringstream format_file(input);
 
@@ -231,6 +232,7 @@ TEST(tracepoint_format_parser, tracepoint_struct_btf)
       "	field:unsigned int fd;	offset:16;	size:8;	signed:0;\n"
       "	field:char * buf;	offset:24;	size:8;	signed:0;\n"
       "	field:size_t count;	offset:32;	size:8;	signed:0;\n"
+      "	field:char comm[TASK_COMM_LEN];	offset:40; size:16; signed:1;\n"
       "\n"
       "print fmt: \"fd: 0x%08lx, buf: 0x%08lx, count: 0x%08lx\", ((unsigned "
       "long)(REC->fd)), ((unsigned long)(REC->buf)), ((unsigned "
@@ -249,6 +251,35 @@ TEST(tracepoint_format_parser, tracepoint_struct_btf)
   EXPECT_THAT(bpftrace.btf_set_, Contains("u64"));
   EXPECT_THAT(bpftrace.btf_set_, Contains("char *"));
   EXPECT_THAT(bpftrace.btf_set_, Contains("size_t"));
+  EXPECT_THAT(bpftrace.btf_set_, Contains("char"));
+  EXPECT_THAT(bpftrace.btf_set_, Contains("TASK_COMM_LEN"));
+}
+
+TEST(tracepoint_format_parser, args_field_access)
+{
+  // Test computing the level of nested structs accessed from tracepoint args
+  BPFtrace bpftrace;
+  Driver driver(bpftrace);
+  ast::TracepointArgsVisitor visitor;
+
+  EXPECT_EQ(driver.parse_str("BEGIN { args->f1->f2->f3 }"), 0);
+  visitor.visit(*driver.root->probes->at(0));
+  EXPECT_EQ(driver.root->probes->at(0)->tp_args_structs_level, 3);
+
+  // Should work via intermediary variable, too
+  EXPECT_EQ(driver.parse_str("BEGIN { $x = args->f1; $x->f2->f3 }"), 0);
+  visitor.visit(*driver.root->probes->at(0));
+  EXPECT_EQ(driver.root->probes->at(0)->tp_args_structs_level, 3);
+
+  // "args" used without field access => level should be 0
+  EXPECT_EQ(driver.parse_str("BEGIN { args }"), 0);
+  visitor.visit(*driver.root->probes->at(0));
+  EXPECT_EQ(driver.root->probes->at(0)->tp_args_structs_level, 0);
+
+  // "args" not used => level should be -1
+  EXPECT_EQ(driver.parse_str("BEGIN { x->f1->f2->f3 }"), 0);
+  visitor.visit(*driver.root->probes->at(0));
+  EXPECT_EQ(driver.root->probes->at(0)->tp_args_structs_level, -1);
 }
 
 } // namespace tracepoint_format_parser
